@@ -20,6 +20,8 @@
 
 #include "chilli.h"
 
+#define _debug_ 1
+
 struct options_t _options;
 
 static select_ctx sctx;
@@ -32,6 +34,12 @@ static int max_requests = 0;
 static redir_request * requests = 0;
 static redir_request * requests_free = 0;
 
+static bstring string_init_reset(bstring s) {
+  if (!s) return bfromcstr("");
+  bassigncstr(s, "");
+  return s;
+}
+
 static redir_request * get_request() {
   redir_request * req = 0;
   int i;
@@ -74,8 +82,22 @@ static redir_request * get_request() {
     return 0;
   }
 
+  req->url  = string_init_reset(req->url);
+  req->data = string_init_reset(req->data);
+  req->post = string_init_reset(req->post);
+  req->wbuf = string_init_reset(req->wbuf);
+  req->hbuf = string_init_reset(req->hbuf);
+
+  /*
+    log_dbg("url->len  %d",req->url->slen);
+    log_dbg("data->len %d",req->data->slen);
+    log_dbg("post->len %d",req->post->slen);
+    log_dbg("wbuf->len %d",req->wbuf->slen);
+  */
+  
   req->state = 0;
   req->next = req->prev = 0;
+  req->proxy = req->headers = 0;
   req->inuse = 1;
   return req;
 }
@@ -86,6 +108,7 @@ static void close_request(redir_request *req) {
   req->proxy = 0;
   req->socket_fd = 0;
   req->state = 0;
+  req->last_active = 0;
   if (requests_free) {
     requests_free->prev = req;
     req->next = requests_free;
@@ -107,6 +130,7 @@ sock_redir_getstate(struct redir_t *redir,
 
   statedir_file(filedest, sizeof(filedest), _options.unixipc, "chilli.ipc");
 
+  memset(&msg, 0, sizeof(msg));
   msg.mtype = REDIR_MSG_STATUS_TYPE;
   memcpy(&msg.mdata.address, address, sizeof(msg.mdata.address));
   memcpy(&msg.mdata.baddress, baddress, sizeof(msg.mdata.baddress));
@@ -172,17 +196,23 @@ static int redir_conn_read(struct conn_t *conn, void *ctx) {
   redir_request *req = (redir_request *)ctx;
   char b[PKT_MAX_LEN];
 
-  int r = safe_read(conn->sock, b, sizeof(b));
+  int r = safe_read(conn->sock, b, sizeof(b)-1);
 
   log_dbg("conn_read: %d", r);
 
   if (r <= 0) {
+
     redir_conn_finish(conn, ctx);
+
   } else if (r > 0) {
-    int w;
+
+    int w = safe_write(req->socket_fd, b, r);
     req->last_active = mainclock_tick();
-    w = safe_write(req->socket_fd, b, r);
+
     log_dbg("write: %d", w);
+    b[r]=0;
+    /*log_dbg("write: [%s]", b);*/
+
     if (r != w) {
       log_err(errno, "problem writing what we read");
       redir_conn_finish(conn, ctx);
@@ -195,7 +225,9 @@ static int
 check_regex(regex_t *re, char *regex, char *s) {
   int ret;
 
+#if(_debug_)
   log_dbg("Checking %s =~ %s", s, regex);
+#endif
   
   if (!re->allocated) {
     if ((ret = regcomp(re, regex, REG_EXTENDED | REG_NOSUB)) != 0) {
@@ -236,12 +268,14 @@ redir_handle_url(struct redir_t *redir,
 	 ! _options.regex_pass_throughs[i].regex_qs[0] )
       break;
 
+#if(_debug_)
     log_dbg("REGEX host=[%s] path=[%s] qs=[%s]",
 	    _options.regex_pass_throughs[i].regex_host,
 	    _options.regex_pass_throughs[i].regex_path,
 	    _options.regex_pass_throughs[i].regex_qs);
 
     log_dbg("Host %s", httpreq->host);
+#endif
 
     if (_options.regex_pass_throughs[i].regex_host[0]) {
       switch(check_regex(&_options.regex_pass_throughs[i].re_host, 
@@ -300,7 +334,6 @@ int redir_accept2(struct redir_t *redir, int idx) {
   struct sockaddr_in baddress;
   socklen_t addrlen;
   char buffer[128];
-  int flags;
 
   addrlen = sizeof(struct sockaddr_in);
 
@@ -309,10 +342,10 @@ int redir_accept2(struct redir_t *redir, int idx) {
 				&addrlen)) < 0) {
     if (errno != ECONNABORTED)
       log_err(errno, "accept()");
-
+    
     return 0;
   }
-
+  
   addrlen = sizeof(struct sockaddr_in);
 
   if (getsockname(new_socket, (struct sockaddr *)&baddress, 
@@ -320,15 +353,8 @@ int redir_accept2(struct redir_t *redir, int idx) {
     log_warn(errno, "getsockname() failed!");
   }
 
-  flags = fcntl(new_socket, F_GETFL, 0);
-  if (flags < 0) flags = 0;
-
-#ifdef O_NDELAY
-  flags |= O_NDELAY;
-#endif
-  
-  if (fcntl(new_socket, F_SETFL, flags) < 0) {
-    log_err(errno, "could not set socket flags");
+  if (ndelay_on(new_socket) < 0) {
+    log_err(errno, "could not set ndelay");
   }
   
   if (idx == 1 && _options.uamui) {
@@ -344,10 +370,10 @@ int redir_accept2(struct redir_t *redir, int idx) {
       return 0; 
     }
     
-    snprintf(buffer,sizeof(buffer)-1,"%s",inet_ntoa(address.sin_addr));
+    safe_snprintf(buffer,sizeof(buffer),"%s",inet_ntoa(address.sin_addr));
     setenv("TCPREMOTEIP",buffer,1);
     setenv("REMOTE_ADDR",buffer,1);
-    snprintf(buffer,sizeof(buffer)-1,"%d",ntohs(address.sin_port));
+    safe_snprintf(buffer,sizeof(buffer),"%d",ntohs(address.sin_port));
     setenv("TCPREMOTEPORT",buffer,1);
     setenv("REMOTE_PORT",buffer,1);
     
@@ -367,8 +393,6 @@ int redir_accept2(struct redir_t *redir, int idx) {
 
     req->uiidx = idx;
     req->socket_fd = new_socket;
-    if (!req->wbuf) req->wbuf = bfromcstr("");
-    else bassigncstr(req->wbuf, "");
 
     conn_set_readhandler(&req->conn, redir_conn_read, req);
     conn_set_donehandler(&req->conn, redir_conn_finish, req);
@@ -417,7 +441,7 @@ int main(int argc, char **argv) {
   
   process_options(argc, argv, 1);
   
-  strncpy(ifr.ifr_name, _options.dhcpif, sizeof(ifr.ifr_name));
+  safe_strncpy(ifr.ifr_name, _options.dhcpif, sizeof(ifr.ifr_name));
   
   if (ioctl(fd, SIOCGIFHWADDR, (caddr_t)&ifr) == 0) {
     memcpy(hwaddr, ifr.ifr_hwaddr.sa_data, PKT_ETH_ALEN);
@@ -500,6 +524,7 @@ int main(int argc, char **argv) {
 	  active++;
 	}
 	
+#if(_debug_ || 1)
 	if (_options.debug) {
 	  struct sockaddr_in address;
 	  socklen_t addrlen = sizeof(address);
@@ -508,31 +533,32 @@ int main(int argc, char **argv) {
 			  &addrlen) >= 0) {
 	    char line[512];
 	    
-	    snprintf(line, sizeof(line),
-		     "#%d (%d) %d connection from %s %d",
-		     timeout ? -1 : active, fd, (int) requests[idx].last_active,
-		     inet_ntoa(address.sin_addr),
-		     ntohs(address.sin_port));
+	    safe_snprintf(line, sizeof(line),
+			  "#%d (%d) %d connection from %s %d",
+			  timeout ? -1 : active, fd, (int) requests[idx].last_active,
+			  inet_ntoa(address.sin_addr),
+			  ntohs(address.sin_port));
 	    
 	    if (requests[idx].conn.sock) {
 	      addrlen = sizeof(address);
 	      if (getpeername(requests[idx].conn.sock, (struct sockaddr *)&address,
 			      &addrlen) >= 0) {
-		snprintf(line+strlen(line), sizeof(line)-strlen(line),
-			 " to %s %d",
-			 inet_ntoa(address.sin_addr),
-			 ntohs(address.sin_port));
+		safe_snprintf(line+strlen(line), sizeof(line)-strlen(line),
+			      " to %s %d",
+			      inet_ntoa(address.sin_addr),
+			      ntohs(address.sin_port));
 	      }
 	    }
 	    
 	    if (timeout) {
-	      snprintf(line+strlen(line), sizeof(line)-strlen(line),
-		       " (timeout)");
+	      safe_snprintf(line+strlen(line), sizeof(line)-strlen(line),
+			    " (timeout)");
 	    }
 
 	    log_dbg("%s", line);
 	  }
 	}
+#endif
       }
     }
 
@@ -592,71 +618,83 @@ int main(int argc, char **argv) {
 	    }
 #endif
 	    
-	    if (net_select_read_fd(&sctx, fd)) {
-	      
-	      if (requests[idx].proxy) {
-		char b[1500];
-		int r;
-		
+	    switch (net_select_read_fd(&sctx, fd)) {
+	    case -1:
+	      {
+		log_dbg("EXCEPTION");
+	      }
+	      break;
+
+	    case 1:
+	      {
+		if (requests[idx].proxy) {
+		  char b[1500];
+		  int r;
+		  
 #ifdef HAVE_SSL
-		if (requests[idx].sslcon) {
-		  /*
-		  log_dbg("proxy_read_ssl");
-		  */
-		  r = openssl_read(requests[idx].sslcon, 
-				   b, sizeof(b), 0);
-		} else
+		  if (requests[idx].sslcon) {
+		    /*
+		      log_dbg("proxy_read_ssl");
+		    */
+		    r = openssl_read(requests[idx].sslcon, 
+				     b, sizeof(b)-1, 0);
+		  } else
 #endif
-		  r = recv(fd, b, sizeof(b), 0);
-		
-		/*
-		log_dbg("proxy_read: %d %d", fd, r);
-		*/
-		
-		if (r <= 0) {
-		  
-		  redir_conn_finish(&requests[idx].conn, &requests[idx]);
-		  
-		} else if (r > 0) {
-		  
-		  int w;
-		  requests[idx].last_active = mainclock_tick();
-		  w = safe_write(requests[idx].conn.sock, b, r);
+		    r = recv(fd, b, sizeof(b)-1, 0);
 		  
 		  /*
-		  log_dbg("proxy_write: %d", w);
-		   */
-		  if (r != w) {
-		    log_err(errno, "problem writing what we read");
+		    log_dbg("proxy_read: %d %d", fd, r);
+		  */
+		  
+		  if (r <= 0) {
+
 		    redir_conn_finish(&requests[idx].conn, &requests[idx]);
+		    
+		  } else if (r > 0) {
+
+		    int w;
+		    requests[idx].last_active = mainclock_tick();
+		    w = safe_write(requests[idx].conn.sock, b, r);
+		    
+		    /*
+		      log_dbg("proxy_write: %d", w);
+		    */
+		    if (r != w) {
+		      log_err(errno, "problem writing what we read");
+		      redir_conn_finish(&requests[idx].conn, &requests[idx]);
+		    }
+		  }
+		  
+		} else {
+#ifdef HAVE_SSL
+		go_again:
+#endif
+		  switch (redir_main(redir, fd, fd, 
+				     &requests[idx].conn.peer,
+				     &requests[idx].baddr, 
+				     requests[idx].uiidx, 
+				     &requests[idx])) {
+		  case 1:
+		    /*log_dbg("redir cont'ed");*/
+#ifdef HAVE_SSL
+		    if (requests[idx].sslcon && 
+			openssl_pending(requests[idx].sslcon) > 0) {
+		      log_dbg("ssl_pending, trying again");
+		      goto go_again;
+		    }
+#endif
+		  break;
+		  case -1: 
+		    log_dbg("redir error");
+		  default:
+		    log_dbg("redir completed");
+		    redir_conn_finish(&requests[idx].conn, &requests[idx]);
+		    break;
 		  }
 		}
-		
-	      } else {
-#ifdef HAVE_SSL
-	      go_again:
-#endif
-		switch (redir_main(redir, fd, fd, 
-				   &requests[idx].conn.peer,
-				   &requests[idx].baddr, 
-				   requests[idx].uiidx, 
-				   &requests[idx])) {
-		case 1:
-#ifdef HAVE_SSL
-		  if (requests[idx].sslcon && 
-		      openssl_pending(requests[idx].sslcon) > 0)
-		    goto go_again;
-#endif
-		  break;
-		case -1: 
-		  log_dbg("redir error");
-		default:
-		  log_dbg("redir completed");
-		  redir_conn_finish(&requests[idx].conn, &requests[idx]);
-		  break;
-		}
 	      }
-	    }
+	      break;
+	    } /* switch(net_select_read_fd()) */
 	  }
 	}
       }
@@ -665,6 +703,8 @@ int main(int argc, char **argv) {
     }
   }
 
+  redir_free(redir);
+
   selfpipe_finish();
   
   return 0;
