@@ -32,6 +32,9 @@ static void destroy_one_ring(net_interface *iface, int what);
 static void setup_filter(net_interface *iface);
 #endif
 
+static int default_sndbuf = 0;
+static int default_rcvbuf = 0;
+
 int safe_accept(int fd, struct sockaddr *sa, socklen_t *lenptr) {
   int ret;
   do {
@@ -56,6 +59,16 @@ int safe_read(int s, void *b, size_t blen) {
   return ret;
 }
 
+int safe_recvfrom(int sockfd, void *buf, size_t len, int flags,
+		  struct sockaddr *src_addr, socklen_t *addrlen) {
+  int ret;
+  do {
+    ret = recvfrom(sockfd, buf, len, flags, src_addr, addrlen);
+  } while (ret == -1 && errno == EINTR);
+  return ret;
+}
+
+
 int safe_write(int s, void *b, size_t blen) {
   int ret;
   do {
@@ -64,6 +77,15 @@ int safe_write(int s, void *b, size_t blen) {
   return ret;
 }
 
+int safe_sendto(int s, const void *b, size_t blen, int flags,
+		const struct sockaddr *dest_addr, socklen_t addrlen) {
+  int ret;
+  do {
+    ret = sendto(s, b, blen, flags, dest_addr, addrlen);
+  } while (ret == -1 && errno == EINTR);
+  return ret;
+}
+
 int safe_close (int fd) {
   int ret;
   do {
@@ -78,8 +100,7 @@ int dev_set_flags(char const *dev, int flags) {
   
   memset(&ifr, 0, sizeof(ifr));
   ifr.ifr_flags = flags;
-  strncpy(ifr.ifr_name, dev, IFNAMSIZ);
-  ifr.ifr_name[IFNAMSIZ-1] = 0; /* Make sure to terminate */
+  safe_strncpy(ifr.ifr_name, dev, IFNAMSIZ);
 
   if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
     log_err(errno,"socket() failed");
@@ -102,8 +123,7 @@ int dev_get_flags(char const *dev, int *flags) {
   int fd;
   
   memset(&ifr, 0, sizeof(ifr));
-  strncpy(ifr.ifr_name, dev, IFNAMSIZ);
-  ifr.ifr_name[IFNAMSIZ-1] = 0; 
+  safe_strncpy(ifr.ifr_name, dev, IFNAMSIZ);
 
   if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
     log_err(errno, "socket() failed");
@@ -140,8 +160,7 @@ int dev_set_address(char const *devname, struct in_addr *address,
   ((struct sockaddr_in *) &ifr.ifr_dstaddr)->sin_len = sizeof (struct sockaddr_in);
 #endif
   
-  strncpy(ifr.ifr_name, devname, IFNAMSIZ);
-  ifr.ifr_name[IFNAMSIZ-1] = 0; /* Make sure to terminate */
+  safe_strncpy(ifr.ifr_name, devname, IFNAMSIZ);
   
   /* Create a channel to the NET kernel. */
   if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
@@ -198,10 +217,10 @@ int dev_set_address(char const *devname, struct in_addr *address,
 }
 
 int net_init(net_interface *netif, char *ifname, uint16_t protocol, int promisc, uint8_t *mac) {
+
   if (ifname) {
     memset(netif, 0, sizeof(net_interface));
-    strncpy(netif->devname, ifname, IFNAMSIZ);
-    netif->devname[IFNAMSIZ] = 0;
+    safe_strncpy(netif->devname, ifname, IFNAMSIZ);
   }
 
   netif->protocol = protocol;
@@ -232,16 +251,54 @@ int net_open(net_interface *netif) {
   return net_open_eth(netif);
 }
 
+static int net_setsockopt(int s, int l, int op, void *v, socklen_t vl) {
+
+  if (setsockopt(s, l, op, v, vl) < 0) {
+    log_err(errno, "setsockopt(s=%d, level=%d, optname=%d, optlen=%d) failed",
+	    s, l, op, (int) vl);
+    return -1;
+  }
+
+  return 0;
+}
+
+
 int net_reopen(net_interface *netif) {
-  net_close(netif);
-  return net_open(netif);
+  int previous_fd = netif->fd;
+  int option;
+  socklen_t len;
+
+  log_dbg("net_reopen(%s)", netif->devname);
+
+  net_open(netif);
+
+  option = (int)(default_sndbuf * 1.1);
+  net_setsockopt(netif->fd, SOL_SOCKET, SO_SNDBUF, &option, sizeof(option));
+  
+  option = (int)(default_rcvbuf * 1.1);
+  net_setsockopt(netif->fd, SOL_SOCKET, SO_RCVBUF, &option, sizeof(option));
+  
+  len = sizeof(default_sndbuf);
+  getsockopt(netif->fd, SOL_SOCKET, SO_SNDBUF, &default_sndbuf, &len);
+  log_dbg("Net SNDBUF %d", default_sndbuf);
+
+  len = sizeof(default_sndbuf);
+  getsockopt(netif->fd, SOL_SOCKET, SO_RCVBUF, &default_rcvbuf, &len);
+  log_dbg("Net RCVBUF %d", default_rcvbuf);
+
+  if (netif->sctx)
+    net_select_rereg(netif->sctx, previous_fd, netif->fd);
+
+  return 0;
 }
 
 int net_close(net_interface *netif) {
 #ifdef USING_PCAP
   if (netif->pd) pcap_close(netif->pd);
+  netif->pd = 0;
 #endif  
   if (netif->fd) close(netif->fd);
+  netif->fd = 0;
   return 0;
 }
 
@@ -282,6 +339,35 @@ int net_select_prepare(select_ctx *sctx) {
   return 0;
 }
 
+int net_select_rereg(select_ctx *sctx, int oldfd, int newfd) {
+  int i;
+  for (i=0; i<sctx->count; i++) {
+    if (sctx->desc[i].fd == oldfd) {
+      sctx->desc[i].fd = newfd;
+#if defined(USING_POLL) && defined(HAVE_SYS_EPOLL_H)
+      {
+	struct epoll_event event;
+	
+	memset(&event, 0, sizeof(event));
+	event.data.fd = oldfd;
+	event.events = EPOLLIN | EPOLLOUT;
+	if (epoll_ctl(sctx->efd, EPOLL_CTL_DEL, oldfd, &event))
+	  log_err(errno, "epoll fd %d not found", oldfd);
+
+	memset(&event, 0, sizeof(event));
+	if (sctx->desc[i].evts & SELECT_READ) event.events |= EPOLLIN;
+	if (sctx->desc[i].evts & SELECT_WRITE) event.events |= EPOLLOUT;
+	event.data.ptr = &sctx->desc[i];
+	if (epoll_ctl(sctx->efd, EPOLL_CTL_ADD, newfd, &event))
+	  log_err(errno, "Failed to watch fd");
+      }
+#endif
+      return 0;
+    }
+  }
+  return -1;
+}
+
 int net_select_reg(select_ctx *sctx, int fd, char evts, 
 		   select_callback cb, void *ctx, int idx) {
   if (!evts) return -3;
@@ -392,12 +478,16 @@ int net_select_fd(select_ctx *sctx, int fd, char evts) {
     sctx->pfds[sctx->count].events |= POLLOUT;
 #endif
 #else
-  if (evts & SELECT_READ)
+  if (evts & SELECT_READ) {
     fd_set(fd, &sctx->rfds);
-  if (evts & SELECT_WRITE)
+    fd_set(fd, &sctx->efds);
+  }
+  if (evts & SELECT_WRITE) {
     fd_set(fd, &sctx->wfds);
-  if (fd > sctx->maxfd) 
+  }
+  if (fd > sctx->maxfd) {
     sctx->maxfd = fd;
+  }
 #endif
   sctx->count++;
   return 0;
@@ -415,8 +505,15 @@ int net_select(select_ctx *sctx) {
   sctx->idleTime.tv_sec = 1;
   sctx->idleTime.tv_usec = 0;
   do {
-    status = select(sctx->maxfd + 1, &sctx->rfds, &sctx->wfds, &sctx->efds, &sctx->idleTime);
+
+    status = select(sctx->maxfd + 1, 
+		    &sctx->rfds, 
+		    &sctx->wfds, 
+		    &sctx->efds, 
+		    &sctx->idleTime);
+
     if (status == -1) net_select_prepare(sctx); /* reset */
+
   } while (status == -1 && errno == EINTR);
 #endif
   return status;
@@ -438,6 +535,8 @@ int net_select_read_fd(select_ctx *sctx, int fd) {
   return 0;
 #endif
 #else
+  if (FD_ISSET(fd, &sctx->efds))
+    return -1;
   return FD_ISSET(fd, &sctx->rfds);
 #endif
 }
@@ -526,7 +625,7 @@ net_pcap_handler(u_char *user, const struct pcap_pkthdr *hdr, const u_char *byte
 #endif
 
 ssize_t 
-net_read_dispatch(net_interface *netif, net_handler func, void *ctx) {
+net_read_dispatch_eth(net_interface *netif, net_handler func, void *ctx) {
 #if defined(USING_PCAP)
   if (netif->pd) {
     struct netpcap np; 
@@ -551,14 +650,22 @@ net_read_dispatch(net_interface *netif, net_handler func, void *ctx) {
 
   {
     uint8_t packet[PKT_MAX_LEN];
-    ssize_t length = net_read(netif, packet, sizeof(packet));
+    ssize_t length = net_read_eth(netif, packet, sizeof(packet));
     if (length <= 0) return length;
     return func(ctx, packet, length);
   }
 }
 
 ssize_t 
-net_read(net_interface *netif, void *d, size_t dlen) {
+net_read_dispatch(net_interface *netif, net_handler func, void *ctx) {
+  uint8_t packet[PKT_MAX_LEN];
+  ssize_t length = safe_read(netif->fd, packet, sizeof(packet));
+  if (length <= 0) return length;
+  return func(ctx, packet, length);
+}
+
+ssize_t 
+net_read_eth(net_interface *netif, void *d, size_t dlen) {
   ssize_t len = 0;
 
 #if defined(USING_PCAP)
@@ -585,30 +692,23 @@ net_read(net_interface *netif, void *d, size_t dlen) {
 
   if (netif->fd) {
 
-    /*len = recvfrom(netif->fd, d, netif->mtu,
-      MSG_DONTWAIT | MSG_TRUNC, NULL, NULL);*/
+    len = safe_recvfrom(netif->fd, d, dlen,
+			MSG_DONTWAIT | MSG_TRUNC, 
+			NULL, NULL);
 
-    len = safe_read(netif->fd, d, dlen);
+    /*len = safe_read(netif->fd, d, dlen);*/
 
     if (len < 0) {
-#ifdef ENETDOWN
-      if (errno == ENETDOWN) {
-	net_reopen(netif);
-      }
-#endif
-      log_err(errno, "read(fd=%d, len=%d, mtu=%d) == %d", netif->fd, dlen, netif->mtu, len);
+      log_err(errno, "net_read_eth(fd=%d, len=%d, mtu=%d) == %d", 
+	      netif->fd, dlen, netif->mtu, len);
       return -1;
     }
   }
-
+  
   return len;
 }
 
-ssize_t net_write(net_interface *netif, void *d, size_t dlen) {
-  return net_write2(netif, d, dlen, 0);
-}
-
-ssize_t net_write2(net_interface *netif, void *d, size_t dlen, struct sockaddr_ll *dest) {
+ssize_t net_write_eth(net_interface *netif, void *d, size_t dlen, struct sockaddr_ll *dest) {
   int fd = netif->fd;
   ssize_t len;
 
@@ -617,30 +717,36 @@ ssize_t net_write2(net_interface *netif, void *d, size_t dlen, struct sockaddr_l
     return tx_ring(netif, d, dlen);
 #endif
 
-  if (dest) {
-    len = sendto(fd, d, dlen, 0, (struct sockaddr *)dest, sizeof(struct sockaddr_ll));
-  } else {
-    len = safe_write(fd, d, dlen);
-  }
-
+  len = safe_sendto(fd, d, dlen, 0, 
+		    (struct sockaddr *)dest, 
+		    sizeof(struct sockaddr_ll));
+  
   if (len < 0) {
-#ifdef ENETDOWN
-    if (errno == ENETDOWN) {
+    switch (errno) {
+    case EWOULDBLOCK:
+      log_err(errno, "packet dropped due to congestion");
       net_reopen(netif);
-    }
-#endif
-#ifdef EMSGSIZE
-    if (errno == EMSGSIZE && dlen > netif->mtu) {
-      net_set_mtu(netif, dlen);
-    }
+      break;
+      
+#ifdef ENETDOWN
+    case ENETDOWN:
+      net_reopen(netif);
+      break;
 #endif
 #ifdef ENXIO
-    if (errno == ENXIO) {
+    case ENXIO:
       net_reopen(netif);
-    }
+      break;
 #endif
-    log_err(errno, "net_write(fd=%d, len=%d) failed", 
-	    netif->fd, dlen);
+#ifdef EMSGSIZE
+    case EMSGSIZE:
+      if (dlen > netif->mtu)
+	net_set_mtu(netif, dlen);
+      break;
+#endif
+    }
+    
+    log_err(errno, "net_write_eth(fd=%d, len=%d) failed", netif->fd, dlen);
     return -1;
   }
 
@@ -651,7 +757,7 @@ int net_set_mtu(net_interface *netif, size_t mtu) {
 #if !defined(USING_PCAP)
   struct ifreq ifr;
   memset(&ifr, 0, sizeof(ifr));
-  strncpy(ifr.ifr_name, netif->devname, sizeof(ifr.ifr_name));
+  safe_strncpy(ifr.ifr_name, netif->devname, sizeof(ifr.ifr_name));
   ifr.ifr_mtu = mtu;
   if (ioctl(netif->fd, SIOCSIFMTU, &ifr) < 0) {
     log_err(errno, "could not set MTU on fd %d",
@@ -826,7 +932,7 @@ int net_open_eth(net_interface *netif) {
   memset(&ifr, 0, sizeof(ifr));
 
   /* Get ifindex */
-  strncpy(ifr.ifr_name, netif->devname, sizeof(ifr.ifr_name));
+  safe_strncpy(ifr.ifr_name, netif->devname, sizeof(ifr.ifr_name));
   if (ioctl(netif->fd, SIOCGIFINDEX, (caddr_t)&ifr) < 0) {
     log_err(errno, "ioctl(SIOCFIGINDEX) failed");
   }
@@ -834,7 +940,7 @@ int net_open_eth(net_interface *netif) {
   netif->ifindex = ifr.ifr_ifindex;
 
   /* Get the MAC address of our interface */
-  strncpy(ifr.ifr_name, netif->devname, sizeof(ifr.ifr_name));
+  safe_strncpy(ifr.ifr_name, netif->devname, sizeof(ifr.ifr_name));
   if (ioctl(netif->fd, SIOCGIFHWADDR, (caddr_t)&ifr) < 0) {
     log_err(errno, "ioctl(d=%d, request=%d) failed", netif->fd, SIOCGIFHWADDR);
     return -1;
@@ -845,7 +951,7 @@ int net_open_eth(net_interface *netif) {
     if ((netif->flags & NET_USEMAC) == 0) {
       memcpy(netif->hwaddr, ifr.ifr_hwaddr.sa_data, PKT_ETH_ALEN);
     } else if (_options.dhcpmacset) {
-      strncpy(ifr.ifr_name, netif->devname, sizeof(ifr.ifr_name));
+      safe_strncpy(ifr.ifr_name, netif->devname, sizeof(ifr.ifr_name));
       memcpy(ifr.ifr_hwaddr.sa_data, netif->hwaddr, PKT_ETH_ALEN);
       if (ioctl(netif->fd, SIOCSIFHWADDR, (caddr_t)&ifr) < 0) {
 	log_err(errno, "ioctl(d=%d, request=%d) failed", netif->fd, SIOCSIFHWADDR);
@@ -892,24 +998,42 @@ int net_open_eth(net_interface *netif) {
   }
 
 #ifndef USING_MMAP
+  /* Let's make this non-blocking */
+  ndelay_on(netif->fd);
+  coe(netif->fd);
+
   option = 1;
-  if (setsockopt(netif->fd, SOL_SOCKET, TCP_NODELAY, &option, sizeof(option)) < 0) {
-    log_err(errno, "setsockopt(s=%d, level=%d, optname=%d, optlen=%d) failed",
-	    netif->fd, SOL_SOCKET, TCP_NODELAY, sizeof(option));
+  if (net_setsockopt(netif->fd, SOL_SOCKET, TCP_NODELAY, &option, sizeof(option)) < 0)
     return -1;
-  }
 
   /* Enable reception and transmission of broadcast frames */
   option = 1;
-  if (setsockopt(netif->fd, SOL_SOCKET, SO_BROADCAST, &option, sizeof(option)) < 0) {
-    log_err(errno, "setsockopt(s=%d, level=%d, optname=%d, optlen=%d) failed",
-	    netif->fd, SOL_SOCKET, SO_BROADCAST, sizeof(option));
+  if (net_setsockopt(netif->fd, SOL_SOCKET, SO_BROADCAST, &option, sizeof(option)) < 0)
     return -1;
+
+  if (_options.sndbuf > 0) {
+    option = _options.sndbuf;
+    net_setsockopt(netif->fd, SOL_SOCKET, SO_SNDBUF, &option, sizeof(option));
+  }
+  
+  if (_options.rcvbuf > 0) {
+    option = _options.rcvbuf;
+    net_setsockopt(netif->fd, SOL_SOCKET, SO_RCVBUF, &option, sizeof(option));
+  }
+
+  {
+    socklen_t len;
+    len = sizeof(default_sndbuf);
+    getsockopt(netif->fd, SOL_SOCKET, SO_SNDBUF, &default_sndbuf, &len);
+    log_dbg("Net SNDBUF %d", default_sndbuf);
+    len = sizeof(default_sndbuf);
+    getsockopt(netif->fd, SOL_SOCKET, SO_RCVBUF, &default_rcvbuf, &len);
+    log_dbg("Net RCVBUF %d", default_rcvbuf);
   }
 #endif
 
   /* Get the MAC address of our interface */
-  strncpy(ifr.ifr_name, netif->devname, sizeof(ifr.ifr_name));
+  safe_strncpy(ifr.ifr_name, netif->devname, sizeof(ifr.ifr_name));
   if (ioctl(netif->fd, SIOCGIFHWADDR, (caddr_t)&ifr) < 0) {
     log_err(errno, "ioctl(d=%d, request=%d) failed", netif->fd, SIOCGIFHWADDR);
     return -1;
@@ -922,7 +1046,7 @@ int net_open_eth(net_interface *netif) {
     if ((netif->flags & NET_USEMAC) == 0) {
       memcpy(netif->hwaddr, ifr.ifr_hwaddr.sa_data, PKT_ETH_ALEN);
     } else if (_options.dhcpmacset) {
-      strncpy(ifr.ifr_name, netif->devname, sizeof(ifr.ifr_name));
+      safe_strncpy(ifr.ifr_name, netif->devname, sizeof(ifr.ifr_name));
       memcpy(ifr.ifr_hwaddr.sa_data, netif->hwaddr, PKT_ETH_ALEN);
       if (ioctl(netif->fd, SIOCSIFHWADDR, (caddr_t)&ifr) < 0) {
 	log_err(errno, "ioctl(d=%d, request=%d) failed", netif->fd, SIOCSIFHWADDR);
@@ -938,7 +1062,7 @@ int net_open_eth(net_interface *netif) {
   /* Get the current interface address, network, and any destination address */
   
   /* Verify that MTU = ETH_DATA_LEN */
-  strncpy(ifr.ifr_name, netif->devname, sizeof(ifr.ifr_name));
+  safe_strncpy(ifr.ifr_name, netif->devname, sizeof(ifr.ifr_name));
   if (ioctl(netif->fd, SIOCGIFMTU, (caddr_t)&ifr) < 0) {
     log_err(errno, "ioctl(d=%d, request=%d) failed", netif->fd, SIOCGIFMTU);
     return -1;
@@ -950,7 +1074,7 @@ int net_open_eth(net_interface *netif) {
   netif->mtu = ifr.ifr_mtu;
   
   /* Get ifindex */
-  strncpy(ifr.ifr_name, netif->devname, sizeof(ifr.ifr_name));
+  safe_strncpy(ifr.ifr_name, netif->devname, sizeof(ifr.ifr_name));
   if (ioctl(netif->fd, SIOCGIFINDEX, (caddr_t)&ifr) < 0) {
     log_err(errno, "ioctl(SIOCFIGINDEX) failed");
   }
@@ -964,7 +1088,7 @@ int net_open_eth(net_interface *netif) {
   if (netif->flags & NET_PROMISC) {
 
     memset(&ifr, 0, sizeof(ifr));
-    strncpy(ifr.ifr_name, netif->devname, sizeof(ifr.ifr_name));
+    safe_strncpy(ifr.ifr_name, netif->devname, sizeof(ifr.ifr_name));
     if (ioctl(netif->fd, SIOCGIFFLAGS, (caddr_t)&ifr) == -1) {
       log_err(errno, "ioctl(SIOCGIFFLAGS)");
     } else {
@@ -978,11 +1102,9 @@ int net_open_eth(net_interface *netif) {
     memset(&mr,0,sizeof(mr));
     mr.mr_ifindex = netif->ifindex;
     mr.mr_type = PACKET_MR_PROMISC;
-    if (setsockopt(netif->fd, SOL_PACKET, PACKET_ADD_MEMBERSHIP, (char *)&mr, sizeof(mr)) < 0) {
-      log_err(errno, "setsockopt(s=%d, level=%d, optname=%d, optlen=%d) failed",
-	      netif->fd, SOL_SOCKET, PACKET_ADD_MEMBERSHIP, sizeof(mr));
+
+    if (net_setsockopt(netif->fd, SOL_PACKET, PACKET_ADD_MEMBERSHIP, (char *)&mr, sizeof(mr)) < 0)
       return -1;
-    }
   }
 #endif
 
@@ -1014,10 +1136,10 @@ int net_open_eth(net_interface *netif) {
   setup_rings2(netif);
   /*setup_filter(netif);*/
 
-  if (_options.sndbuf)
+  if (_options.sndbuf > 0)
     set_buffer(netif, SO_SNDBUF, _options.sndbuf * 1024);
 
-  if (_options.rcvbuf)
+  if (_options.rcvbuf > 0)
     set_buffer(netif, SO_RCVBUF, _options.rcvbuf * 1024);
 #endif
 
@@ -1101,7 +1223,7 @@ int net_open_eth(net_interface *netif) {
 
   /* Find suitable device */
   for (devnum = 0; devnum < 255; devnum++) { /* TODO 255 */ 
-    snprintf(devname, sizeof(devname), "/dev/bpf%d", devnum);
+    chilli_snprintf(devname, sizeof(devname), "/dev/bpf%d", devnum);
     devname[sizeof(devname)] = 0;
     if ((netif->fd = open(devname, O_RDWR)) >= 0) break;
     if (errno != EBUSY) break;
@@ -1113,7 +1235,7 @@ int net_open_eth(net_interface *netif) {
 
   /* Set the interface */
   memset(&ifr, 0, sizeof(ifr));
-  strncpy(ifr.ifr_name, netif->devname, sizeof(ifr.ifr_name));
+  safe_strncpy(ifr.ifr_name, netif->devname, sizeof(ifr.ifr_name));
   if (ioctl(netif->fd, BIOCSETIF, &ifr) < 0) {
     log_err(errno,"ioctl() failed");
     return -1;
@@ -1394,7 +1516,7 @@ static void setup_one_ring(net_interface *iface, unsigned ring_size, int mtu, in
     
     req.tp_frame_nr = (req.tp_block_size / req.tp_frame_size) * req.tp_block_nr;
     
-    ret = setsockopt(iface->fd, SOL_PACKET, what, &req, sizeof(req));
+    ret = net_setsockopt(iface->fd, SOL_PACKET, what, &req, sizeof(req));
     if (!ret)
       break;
 
@@ -1424,7 +1546,7 @@ static void destroy_one_ring(net_interface *iface, int what) {
   ring = what == PACKET_RX_RING ? &iface->rx_ring : &iface->tx_ring;
   
   memset(&req, 0, sizeof(req));
-  setsockopt(iface->fd, SOL_PACKET, what, &req, sizeof(req));
+  net_setsockopt(iface->fd, SOL_PACKET, what, &req, sizeof(req));
   free(ring->frames);
   memset(ring, 0, sizeof(*ring));
 }
@@ -1464,7 +1586,7 @@ static void setup_rings(net_interface *iface, unsigned size, int mtu) {
   
   /* We want version 2 ring buffers to avoid 64-bit uncleanness */
   val = TPACKET_V2;
-  ret = setsockopt(iface->fd, SOL_PACKET, PACKET_VERSION, &val, sizeof(val));
+  ret = net_setsockopt(iface->fd, SOL_PACKET, PACKET_VERSION, &val, sizeof(val));
 
   if (ret) {
     log_err(errno, "Failed to set version 2 ring buffer format");
@@ -1484,7 +1606,7 @@ static void setup_rings(net_interface *iface, unsigned size, int mtu) {
   
   /* Drop badly formatted packets */
   val = 1;
-  ret = setsockopt(iface->fd, SOL_PACKET, PACKET_LOSS, &val, sizeof(val));
+  ret = net_setsockopt(iface->fd, SOL_PACKET, PACKET_LOSS, &val, sizeof(val));
 
   if (ret)
     log_err(errno, "Failed to set packet drop mode");
@@ -1538,7 +1660,7 @@ static void set_buffer(net_interface *iface, int what, int size) {
   socklen_t len;
   int ret, val;
   
-  ret = setsockopt(iface->fd, SOL_SOCKET, what, &size, sizeof(size));
+  ret = net_setsockopt(iface->fd, SOL_SOCKET, what, &size, sizeof(size));
   if (ret) {
     log_err(errno, "Failed to set the %s buffer size",
 	    what == SO_SNDBUF ? "send" : "receive");
@@ -1599,7 +1721,7 @@ static void setup_filter(net_interface *iface) {
       .len = sizeof(filter) / sizeof(struct sock_filter)
     };
     
-    if (setsockopt(iface->fd, SOL_SOCKET, SO_ATTACH_FILTER, &prog, sizeof(prog)))
+    if (net_setsockopt(iface->fd, SOL_SOCKET, SO_ATTACH_FILTER, &prog, sizeof(prog)))
       log_err(errno, "Failed to set up the socket filter");
   }
 }
