@@ -45,41 +45,44 @@ void radius_addcalledstation(struct radius_t *radius, struct radius_packet_t *pa
   if (_options.nasmac)
     mac = (uint8_t *)_options.nasmac;
   else 
-    snprintf((char*)(mac = b), sizeof(b), "%.2X-%.2X-%.2X-%.2X-%.2X-%.2X", 
-	     radius->nas_hwaddr[0], radius->nas_hwaddr[1], radius->nas_hwaddr[2],
-	     radius->nas_hwaddr[3], radius->nas_hwaddr[4], radius->nas_hwaddr[5]);
+    safe_snprintf((char*)(mac = b), sizeof(b), "%.2X-%.2X-%.2X-%.2X-%.2X-%.2X", 
+		  radius->nas_hwaddr[0], radius->nas_hwaddr[1], radius->nas_hwaddr[2],
+		  radius->nas_hwaddr[3], radius->nas_hwaddr[4], radius->nas_hwaddr[5]);
   
   radius_addattr(radius, pack, RADIUS_ATTR_CALLED_STATION_ID, 0, 0, 0, mac, strlen((char*)mac)); 
 }
 
-#if(deeplog)
-int radius_printqueue(struct radius_t *this) {
+int radius_printqueue(int fd, struct radius_t *this) {
+  char line[1024];
   int mx = 256;
   int n;
 
   if (this->qsize)
     mx = this->qsize;
 
-  log_dbg("next %d, first %d, last %d", 
-	 this->qnext, this->first, this ->last);
-
+  safe_snprintf(line, sizeof(line), "next %d, first %d, last %d\n", 
+		this->qnext, this->first, this->last);
+  
+  safe_write(fd, line, strlen(line));
+  
   for(n=0; n < mx; n++) {
     if (this->queue[n].state) {
-      log_dbg("n=%3d id=%3d state=%3d next=%3d prev=%3d %8d %8d %d",
-	      n, 
-	      this->queue[n].p.id,
-	      this->queue[n].state,
-	      this->queue[n].next,
-	      this->queue[n].prev,
-	      (int) this->queue[n].timeout.tv_sec,
-	      (int) this->queue[n].timeout.tv_usec,
-	      (int) this->queue[n].retrans);
+      safe_snprintf(line, sizeof(line), 
+		    "n=%3d id=%3d state=%3d next=%3d prev=%3d %8d %8d %d\n",
+		    n, 
+		    this->queue[n].p.id,
+		    this->queue[n].state,
+		    this->queue[n].next,
+		    this->queue[n].prev,
+		    (int) this->queue[n].timeout.tv_sec,
+		    (int) this->queue[n].timeout.tv_usec,
+		    (int) this->queue[n].retrans);
+      safe_write(fd, line, strlen(line));
     }
   }
-
+  
   return 0;
 }
-#endif
 
 /* 
  * radius_hmac_md5()
@@ -204,7 +207,7 @@ int radius_queue_in(struct radius_t *this, struct radius_packet_t *pack,
 
 #if(deeplog)
   if (_options.debug) {
-    radius_printqueue(this);
+    radius_printqueue(0, this);
   }
 #endif
 
@@ -282,7 +285,7 @@ int radius_queue_in(struct radius_t *this, struct radius_packet_t *pack,
     log_dbg("sending radius packet (code=%d, id=%d, len=%d)\n",
 	    pack->code, pack->id, ntohs(pack->length));
 
-    radius_printqueue(this);
+    radius_printqueue(0, this);
   }
 #endif
 
@@ -335,7 +338,7 @@ radius_queue_out(struct radius_t *this, struct radius_packet_t *pack,
 #if(deeplog)
   if (_options.debug) {
     log_dbg("radius_queue_out");
-    radius_printqueue(this);
+    radius_printqueue(0, this);
   }
 #endif
   
@@ -358,7 +361,7 @@ radius_queue_out(struct radius_t *this, struct radius_packet_t *pack,
 #if(deeplog)
   if (_options.debug) {
     log_dbg("radius_queue_out end");
-    radius_printqueue(this);
+    radius_printqueue(0, this);
   }
 #endif
 
@@ -380,7 +383,7 @@ static int radius_queue_reschedule(struct radius_t *this, int idx) {
 #if(deeplog)
   if (_options.debug) {
     log_dbg("radius_reschedule");
-    radius_printqueue(this);
+    radius_printqueue(0, this);
   }
 #endif
 
@@ -418,7 +421,7 @@ static int radius_queue_reschedule(struct radius_t *this, int idx) {
   
 #if(deeplog)
   if (_options.debug) {
-    radius_printqueue(this);
+    radius_printqueue(0, this);
   }
 #endif
   
@@ -568,7 +571,7 @@ int radius_timeout(struct radius_t *this) {
   if (_options.debug) {
     log_dbg("radius_timeout(%d) %8d %8d", this->first, 
 	    (int)now.tv_sec, (int)now.tv_usec);
-    radius_printqueue(this);
+    radius_printqueue(0, this);
   }
 #endif
   
@@ -645,7 +648,7 @@ int radius_timeout(struct radius_t *this) {
 	     (int) this->queue[this->first].timeout.tv_sec, 
 	     (int) this->queue[this->first].timeout.tv_usec); 
     }
-    radius_printqueue(this);
+    radius_printqueue(0, this);
   }
 #endif
 
@@ -803,7 +806,7 @@ radius_getnextattr(struct radius_packet_t *pack, struct radius_attr_t **attr,
   */
 
   while (offset < len) {
-    t = (struct radius_attr_t *)(&pack->payload[offset]);
+    t = (struct radius_attr_t *) (&pack->payload[offset]);
 
     /*
     if (0) {
@@ -812,9 +815,9 @@ radius_getnextattr(struct radius_packet_t *pack, struct radius_attr_t **attr,
     }
     */
 
-    offset +=  t->l;
+    offset += t->l;
 
-    if (t->t == 0)
+    if (t->t == 0 || t->l < 2)
       return -1;
     
     if (t->t != type) 
@@ -846,31 +849,6 @@ radius_getnextattr(struct radius_packet_t *pack, struct radius_attr_t **attr,
   return -1; /* Not found */
 }
 
-/* 
- * radius_countattr()
- * Count the number of instances of an attribute in a packet.
- */
-int 
-radius_countattr(struct radius_packet_t *pack, uint8_t type) {
-  struct radius_attr_t *t;
-  size_t offset = 0;
-  int count = 0;
-  
-  /* Need to check pack -> length */
-  
-  do {
-    t = (struct radius_attr_t*)(&pack->payload[offset]);
-    if (t->t == type) {
-      count++;
-    }
-    offset +=  2 + t->l;
-  } while (offset < ntohs(pack->length));
-  
-  /*if (0) printf("Count %d\n", count);*/
-  return count;
-}
-
-
 /* 
  * radius_cmpattr()
  * Compare two attributes to see if they are the same.
@@ -1122,30 +1100,6 @@ int radius_pwencode(struct radius_t *this,
 
   log_dbg("pw encode secret=%s", secret);
 
-  /*
-  if (_options.debug) {
-    char out[1024];
-    out[0]=0;
-
-    log_dbg("pwencode src %s",src);
-    for (n=0; n< srclen; n++) {
-      sprintf(out + strlen(out), "%.2x ", src[n]);
-      if ((n % 16) == 15)
-	sprintf(out+strlen(out),"\n");
-    }
-    log_dbg("%s", out);
-
-    out[0]=0;
-    log_dbg("pwencode authenticator");
-    for (n=0; n< RADIUS_AUTHLEN; n++) {
-      sprintf(out+strlen(out),"%.2x ", authenticator[n]);
-      if ((n % 16) == 15)
-	sprintf(out+strlen(out),"\n");
-    }
-    log_dbg("%s", out);
-  }
-  */
-
   memset(dst, 0, dstsize);
 
   /* Make dstlen multiple of 16 */
@@ -1343,7 +1297,7 @@ void radius_set(struct radius_t *this, unsigned char *hwaddr, int debug) {
     this->hisaddr1.s_addr = this->hisaddr0.s_addr;
 
     this->secretlen = 6;
-    strncpy(this->secret, "radsec", sizeof(this->secret));
+    safe_strncpy(this->secret, "radsec", sizeof(this->secret));
   } else {
     this->hisaddr0.s_addr = _options.radiusserver1.s_addr;
     this->hisaddr1.s_addr = _options.radiusserver2.s_addr;
@@ -1576,8 +1530,10 @@ radius_default_pack(struct radius_t *this,
   }
 
   switch (code) {
-  case RADIUS_CODE_ACCESS_REJECT:
   case RADIUS_CODE_ACCESS_ACCEPT:
+  case RADIUS_CODE_ACCESS_REJECT:
+    break;
+
   case RADIUS_CODE_ACCESS_REQUEST:
   case RADIUS_CODE_ACCOUNTING_REQUEST:
 
@@ -1601,6 +1557,10 @@ radius_default_pack(struct radius_t *this,
 		     RADIUS_VENDOR_CHILLISPOT,
 		     RADIUS_ATTR_CHILLISPOT_ACCT_VIEW_POINT, 
 		     v, 0, 0);
+
+      radius_addattr(this, pack, RADIUS_ATTR_EVENT_TIMESTAMP, 0, 0, 
+		     mainclock_wall(), NULL, 0); 
+      
     }
     break;
 
