@@ -32,6 +32,8 @@
 #define inaddr(x)    (((struct sockaddr_in *)&ifr->x)->sin_addr)
 #define inaddr2(p,x) (((struct sockaddr_in *)&(p)->x)->sin_addr)
 
+#define _debug_ 1
+
 int tun_discover(struct tun_t *this) {
   net_interface netif;
   struct ifconf ic;
@@ -72,7 +74,7 @@ int tun_discover(struct tun_t *this) {
     memset(&netif, 0, sizeof(netif));
 
     /* device name and address */
-    strncpy(netif.devname, ifr->ifr_name, sizeof(netif.devname));
+    safe_strncpy(netif.devname, ifr->ifr_name, sizeof(netif.devname));
     netif.address = inaddr(ifr_addr);
 
     log_dbg("Interface: %s", ifr->ifr_name);
@@ -249,8 +251,7 @@ int tun_gifindex(struct tun_t *this, uint32_t *index) {
   ifr.ifr_addr.sa_family = AF_INET;
   ifr.ifr_dstaddr.sa_family = AF_INET;
   ifr.ifr_netmask.sa_family = AF_INET;
-  strncpy(ifr.ifr_name, tuntap(this).devname, IFNAMSIZ);
-  ifr.ifr_name[IFNAMSIZ-1] = 0; /* Make sure to terminate */
+  safe_strncpy(ifr.ifr_name, tuntap(this).devname, IFNAMSIZ);
   if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
     log_err(errno, "socket() failed");
   }
@@ -382,8 +383,7 @@ int tun_addaddr(struct tun_t *this, struct in_addr *addr,
   memset(&areq, 0, sizeof(areq));
 
   /* Set up interface name */
-  strncpy(areq.ifra_name, tuntap(this).devname, IFNAMSIZ);
-  areq.ifra_name[IFNAMSIZ-1] = 0; /* Make sure to terminate */
+  safe_strncpy(areq.ifra_name, tuntap(this).devname, IFNAMSIZ);
 
   ((struct sockaddr_in*) &areq.ifra_addr)->sin_family = AF_INET;
   ((struct sockaddr_in*) &areq.ifra_addr)->sin_len = sizeof(areq.ifra_addr);
@@ -466,10 +466,13 @@ int tuntap_interface(struct _net_interface *netif) {
 
 #if defined(__linux__)
   /* Open the actual tun device */
-  if ((netif->fd  = open("/dev/net/tun", O_RDWR)) < 0) {
+  if ((netif->fd = open("/dev/net/tun", O_RDWR)) < 0) {
     log_err(errno, "open() failed");
     return -1;
   }
+
+  ndelay_on(netif->fd);
+  coe(netif->fd);
   
   /* Set device flags. For some weird reason this is also the method
      used to obtain the network interface name */
@@ -478,14 +481,25 @@ int tuntap_interface(struct _net_interface *netif) {
 
   /* Tun device, no packet info */
   ifr.ifr_flags = (_options.usetap ? IFF_TAP : IFF_TUN) | IFF_NO_PI; 
-
-#if defined(IFF_ONE_QUEUE) && defined(SIOCSIFTXQLEN)
-  ifr.ifr_flags |= IFF_ONE_QUEUE;
+  
+  ifr.ifr_flags = ifr.ifr_flags
+#ifdef IFF_MULTICAST
+    | IFF_MULTICAST
 #endif
+#ifdef IFF_BROADCAST
+    | IFF_BROADCAST
+#endif
+#ifdef IFF_PROMISC
+    | IFF_PROMISC
+#endif
+#if defined(IFF_ONE_QUEUE) && defined(SIOCSIFTXQLEN)
+    | IFF_ONE_QUEUE
+#endif
+    ;
 
   if (_options.tundev && *_options.tundev && 
       strcmp(_options.tundev, "tap") && strcmp(_options.tundev, "tun"))
-    strncpy(ifr.ifr_name, _options.tundev, IFNAMSIZ);
+    safe_strncpy(ifr.ifr_name, _options.tundev, IFNAMSIZ);
 
   if (ioctl(netif->fd, TUNSETIFF, (void *) &ifr) < 0) {
     log_err(errno, "ioctl() failed");
@@ -499,7 +513,7 @@ int tuntap_interface(struct _net_interface *netif) {
     int nfd;
     memset(&nifr, 0, sizeof(nifr));
     if ((nfd = socket (AF_INET, SOCK_DGRAM, 0)) >= 0) {
-      strncpy(nifr.ifr_name, ifr.ifr_name, IFNAMSIZ);
+      safe_strncpy(nifr.ifr_name, ifr.ifr_name, IFNAMSIZ);
       nifr.ifr_qlen = _options.txqlen;
       
       if (ioctl(nfd, SIOCSIFTXQLEN, (void *) &nifr) >= 0) 
@@ -514,8 +528,7 @@ int tuntap_interface(struct _net_interface *netif) {
   }
 #endif
   
-  strncpy(netif->devname, ifr.ifr_name, IFNAMSIZ);
-  netif->devname[IFNAMSIZ-1] = 0;
+  safe_strncpy(netif->devname, ifr.ifr_name, IFNAMSIZ);
   
   ioctl(netif->fd, TUNSETNOCSUM, 1); /* Disable checksums */
 
@@ -525,7 +538,7 @@ int tuntap_interface(struct _net_interface *netif) {
     netif->flags |= NET_ETHHDR;
     if ((fd = socket (AF_INET, SOCK_DGRAM, 0)) >= 0) {
       memset(&ifr, 0, sizeof(ifr));
-      strncpy(ifr.ifr_name, netif->devname, IFNAMSIZ);
+      safe_strncpy(ifr.ifr_name, netif->devname, IFNAMSIZ);
       if (ioctl(fd, SIOCGIFHWADDR, &ifr) < 0) {
 	log_err(errno, "ioctl(d=%d, request=%d) failed", fd, SIOCGIFHWADDR);
       }
@@ -543,7 +556,7 @@ int tuntap_interface(struct _net_interface *netif) {
 
   /* Find suitable device */
   for (devnum = 0; devnum < 255; devnum++) { /* TODO 255 */ 
-    snprintf(devname, sizeof(devname), "/dev/tun%d", devnum);
+    safe_snprintf(devname, sizeof(devname), "/dev/tun%d", devnum);
     if ((netif->fd = open(devname, O_RDWR)) >= 0) break;
     if (errno != EBUSY) break;
   } 
@@ -552,7 +565,7 @@ int tuntap_interface(struct _net_interface *netif) {
     return -1;
   }
 
-  snprintf(netif->devname, sizeof(netif->devname), "tun%d", devnum);
+  safe_snprintf(netif->devname, sizeof(netif->devname), "tun%d", devnum);
 
   /* The tun device we found might have "old" IP addresses allocated */
   /* We need to delete those. This problem is not present on Linux */
@@ -560,7 +573,7 @@ int tuntap_interface(struct _net_interface *netif) {
   memset(&areq, 0, sizeof(areq));
 
   /* Set up interface name */
-  strncpy(areq.ifra_name, netif->devname, sizeof(areq.ifra_name));
+  safe_strncpy(areq.ifra_name, netif->devname, sizeof(areq.ifra_name));
 
   /* Create a channel to the NET kernel. */
   if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
@@ -615,11 +628,11 @@ int tuntap_interface(struct _net_interface *netif) {
 
   close (if_fd);
   
-  snprintf(netif->devname, sizeof(netif->devname), "tun%d", ppa);
-  netif->devname[sizeof(netif->devname)-1] = 0;
+  safe_snprintf(netif->devname, sizeof(netif->devname),
+		"tun%d", ppa);
 
   memset(&ifr, 0, sizeof(ifr));
-  strcpy(ifr.ifr_name, netif->devname);
+  safe_strncpy(ifr.ifr_name, netif->devname, sizeof(ifr.ifr_name));
   ifr.ifr_ip_muxid = muxid;
   
   if (ioctl(ip_fd, SIOCSIFMUXID, &ifr) < 0) {
@@ -683,43 +696,66 @@ struct tundecap {
 
 static int tun_decaps_cb(void *ctx, void *packet, size_t length) {
   struct tundecap *c = (struct tundecap *)ctx;
+  struct pkt_iphdr_t *iph;
+  int ethsize = 0;
 
-  /*
-  if (_options.debug && c->idx == 0)
-    log_dbg("tun_decaps(idx=%d, len=%d)", tun(c->this,c->idx).ifindex, length);
-  */
+  char ethhdr = (tun(c->this, c->idx).flags & NET_ETHHDR) != 0;
+
+#if(_debug_)
+  log_dbg("tun_decaps(idx=%d, len=%d)", tun(c->this, c->idx).ifindex, length);
+#endif
+
+  if (length < PKT_IP_HLEN)
+    return -1;
+
+  if (ethhdr) {
+    
+    if (length < PKT_IP_HLEN + PKT_ETH_HLEN)
+      return -1;
+    
+    ethsize = PKT_ETH_HLEN;
+    iph = iphdr(packet);
+    
+  } else {
+    
+    iph = (struct pkt_iphdr_t *)packet;
+    
+  }
 
 #if defined(HAVE_NETFILTER_QUEUE) || defined(HAVE_NETFILTER_COOVA)
   if (_options.uamlisten.s_addr != _options.dhcplisten.s_addr) {
-    struct pkt_iphdr_t *iph;
-
-    int ethhdr = (tun(c->this, c->idx).flags & NET_ETHHDR) != 0;
-
-    if (ethhdr)
-      iph = (struct pkt_iphdr_t *)((char *)packet + PKT_ETH_HLEN);
-    else
-      iph = (struct pkt_iphdr_t *)packet;
-
     iph->daddr  = iph->daddr & ~(_options.mask.s_addr);
     iph->daddr |= _options.dhcplisten.s_addr & _options.mask.s_addr;
-
     chksum(iph);
   }
 #endif
 
   if (c->idx > 0) {
-    struct pkt_iphdr_t *iph = iphdr(packet);
 #ifdef ENABLE_NETNAT
     struct pkt_udphdr_t *udph = udphdr(packet);
     if (iph->daddr == tun(c->this, c->idx).address.s_addr &&
 	ntohs(udph->dst) > 10000 && ntohs(udph->dst) < 30000) {
       if (nat_undo(c->this, c->idx, packet, length))
 	return -1;
-    } else 
+    }
 #endif
     if ((iph->daddr & _options.mask.s_addr) != _options.net.s_addr) {
       return -1;
     }
+  } 
+  
+  if (iph->version_ihl != PKT_IP_VER_HLEN) {
+#if(_debug_)
+    log_dbg("dropping non-IPv4");
+#endif
+    return -1;
+  }
+
+  if ((int)ntohs(iph->tot_len) + ethsize > length) {
+    log_dbg("dropping ip packet; ip-len=%d + eth-hdr=%d > read-len=%d",
+	    (int)ntohs(iph->tot_len),
+	    ethsize, (int)length);
+    return -1;
   }
 
   return c->this->cb_ind(c->this, packet, length, c->idx);
@@ -730,26 +766,26 @@ int tun_decaps(struct tun_t *this, int idx) {
 #if defined(__linux__) 
   ssize_t length;
   struct tundecap c;
-
+  
   c.this = this;
   c.idx = idx;
   
   if ((length = net_read_dispatch(&tun(this, idx), tun_decaps_cb, &c)) < 0) 
     return -1;
-
+  
   return length;
-
+  
 #elif defined (__FreeBSD__) || defined (__APPLE__) || defined (__OpenBSD__) || defined (__NetBSD__)
   unsigned char buffer[PACKET_MAX];
   ssize_t status;
-
-  if ((status = net_read(&tun(this, idx), buffer, sizeof(buffer))) <= 0) {
+  
+  if ((status = safe_read(&tun(this, idx).fd, buffer, sizeof(buffer))) <= 0) {
     log_err(errno, "read() failed");
     return -1;
   }
-
+  
   /*
-  log_dbg("tun_decaps(%d) %s",status,tun(tun,idx).devname);
+    log_dbg("tun_decaps(%d) %s",status,tun(tun,idx).devname);
   */
 
    if (this->cb_ind)
@@ -795,14 +831,14 @@ int tun_write(struct tun_t *tun, uint8_t *pack, size_t len, int idx) {
   *((uint32_t *)(&buffer))=htonl(AF_INET);
   memcpy(&buffer[4], pack, len);
 
-  return net_write(&tun(tun, idx), buffer, len+4);
+  return safe_write(tun(tun, idx).fd, buffer, len+4);
 
 #elif defined(__linux__) || defined (__FreeBSD__) || defined (__APPLE__) || defined (__NetBSD__)
 
-  if (idx > 0) {
-  }
+  /*if (idx > 0) {
+    }*/
 
-  return net_write(&tun(tun, idx), pack, len);
+  return safe_write(tun(tun, idx).fd, pack, len);
 
 #elif defined (__sun__)
 
@@ -815,6 +851,7 @@ int tun_write(struct tun_t *tun, uint8_t *pack, size_t len, int idx) {
 }
 
 int tun_encaps(struct tun_t *tun, uint8_t *pack, size_t len, int idx) {
+  int result;
 
   if (_options.tcpwin)
     pkt_shape_tcpwin(pack, &len);
@@ -892,7 +929,13 @@ int tun_encaps(struct tun_t *tun, uint8_t *pack, size_t len, int idx) {
 
   /*log_dbg("tun_encaps(%s) len=%d", tun(tun,idx).devname, len);*/
 
-  return tun_write(tun, pack, len, idx);
+  result = tun_write(tun, pack, len, idx);
+
+  if (result < 0) {
+    log_err(errno, "tun_write(%d)", result);
+  }
+
+  return result;
 }
 
 int tun_runscript(struct tun_t *tun, char* script, int wait) {
@@ -936,34 +979,31 @@ int tun_runscript(struct tun_t *tun, char* script, int wait) {
     exit(0);
   }
 
-  strncpy(saddr, inet_ntoa(tuntap(tun).address), sizeof(saddr));
-  saddr[sizeof(saddr)-1] = 0;
+  safe_strncpy(saddr, inet_ntoa(tuntap(tun).address), sizeof(saddr));
   if (setenv("ADDR", saddr, 1 ) != 0) {
     log_err(errno, "setenv() did not return 0!");
     exit(0);
   }
 
-  strncpy(smask, inet_ntoa(tuntap(tun).netmask), sizeof(smask));
-  smask[sizeof(smask)-1] = 0;
+  safe_strncpy(smask, inet_ntoa(tuntap(tun).netmask), sizeof(smask));
   if (setenv("MASK", smask, 1) != 0) {
     log_err(errno, "setenv() did not return 0!");
     exit(0);
   }
 
-  strncpy(b, inet_ntoa(net), sizeof(b));
-  b[sizeof(b)-1] = 0;
+  safe_strncpy(b, inet_ntoa(net), sizeof(b));
   if (setenv("NET", b, 1 ) != 0) {
     log_err(errno, "setenv() did not return 0!");
     exit(0);
   }
 
-  snprintf(b, sizeof(b), "%d", _options.uamport);
+  safe_snprintf(b, sizeof(b), "%d", _options.uamport);
   if (setenv("UAMPORT", b, 1 ) != 0) {
     log_err(errno, "setenv() did not return 0!");
     exit(0);
   }
 
-  snprintf(b, sizeof(b), "%d", _options.uamuiport);
+  safe_snprintf(b, sizeof(b), "%d", _options.uamuiport);
   if (setenv("UAMUIPORT", b, 1 ) != 0) {
     log_err(errno, "setenv() did not return 0!");
     exit(0);
